@@ -95,7 +95,7 @@ class PatchwiseBatchGenerator(Sequence):
         x_batch = np.zeros(
             (
                 self.batch_size,
-                self.config["IMAGE_W"],
+                self.config["IMAGE_H"],
                 self.config["IMAGE_W"],
                 self.imgdepth,
             ),
@@ -139,11 +139,12 @@ class PatchwiseBatchGenerator(Sequence):
             patch_x = patch[1]
             patch_y = patch[2]
 
-            imgw, imgh = imagereader.read_width_height(
-                self.images[img_index]["filename"]
-            )
             tile = None
             if self.config["NUM_PATCHES"] > 1:
+                imgw, imgh = imagereader.read_width_height(
+                    self.images[img_index]["filename"]
+                )
+
                 tile = imagereader.get_tile_coordinates(
                     imgw=imgw,
                     imgh=imgh,
@@ -239,14 +240,58 @@ class PatchwiseBatchGenerator(Sequence):
 
     def aug_image(self, train_instance, jitter, region=None):
 
+
+        def resize(image,shape,copy=False):
+            image = np.array(
+                Image.fromarray(image).resize(
+                    shape,
+                    resample=Image.BILINEAR,
+                ),
+                copy=copy
+            )
+
+            return image
+
         if "filename_filtered" in train_instance:
             image_name = train_instance["filename_filtered"]
         else:
             image_name = train_instance["filename"]
-
+        patch_mode = region is not None
         image = imagereader.image_read(image_name, region)
         h = image.shape[0]
         w = image.shape[1]
+
+        # if the filtered image size is not equals the input image size
+        # that means we are in mixed-size mode. Set region randomly
+        if h != self.config["IMAGE_H"] or w != self.config["IMAGE_W"]:
+            diffh = h - self.config["IMAGE_H"]
+            diffw = w - self.config["IMAGE_W"]
+            if diffh > 0.0:
+                starth = np.random.randint(0, diffh)
+            else:
+                starth = 0
+            if diffw > 0.0:
+                startw = np.random.randint(0, diffw)
+            else:
+                startw = 0
+            endh = starth+self.config["IMAGE_H"]
+            endw = startw+self.config["IMAGE_W"]
+            region = np.s_[starth:endh, startw:endw]
+            image = image[region]
+
+            # Format region into original coordinates
+            original_width, original_height = train_instance["img_size"]
+            endh_scaled = endh*original_height/h
+            starth_scaled = starth*original_height/h
+
+            endw_scaled = endw * original_width / w
+            startw_scaled = startw * original_width / w
+            region = np.s_[starth_scaled:endh_scaled, startw_scaled:endw_scaled]
+
+            # w and h have to be updated now!
+            w = endw-startw
+            h = endh-starth
+
         all_objs = copy.deepcopy(train_instance["object"])
 
         if jitter:
@@ -254,13 +299,7 @@ class PatchwiseBatchGenerator(Sequence):
             # scale the image
 
             scale = np.random.uniform() / 10.0 + 1.0
-            # scale = 1.0
-            image = np.array(
-                Image.fromarray(image).resize(
-                    (int(image.shape[1] * scale), int(image.shape[0] * scale)),
-                    resample=Image.BILINEAR,
-                )
-            )
+            image = resize(image, (int(image.shape[1] * scale), int(image.shape[0] * scale)), copy=True)
 
             # translate the image
             max_offx = (scale - 1.0) * w
@@ -270,18 +309,17 @@ class PatchwiseBatchGenerator(Sequence):
 
             image = image[offy : (offy + h), offx : (offx + w)]
 
-            image = np.array(
-                Image.fromarray(image).resize(
-                    (self.config["IMAGE_H"], self.config["IMAGE_W"]),
-                    resample=Image.BILINEAR,
-                )
-            )
+            image = resize(image, (self.config["IMAGE_W"], self.config["IMAGE_H"]),copy=True)
+
 
             # flip the image
             flip_selection = np.random.randint(0, 4)
-            flip_vertical = flip_selection == 1
+
+
+            flip_both = flip_selection == 1
             flip_horizontal = flip_selection == 2
-            flip_both = flip_selection == 3
+            flip_vertical = flip_selection == 3
+
 
             if flip_vertical:
                 image = np.flip(image, 1)
@@ -290,45 +328,44 @@ class PatchwiseBatchGenerator(Sequence):
             if flip_both:
                 image = np.flip(np.flip(image, 0), 1)
 
-            num_rots = np.random.randint(4)
-            image = np.rot90(image, k=num_rots, axes=(1, 0))
+            if image.shape[0] == image.shape[1]:
+                num_rots = np.random.randint(4)
+            else:
+                num_rots = np.random.choice([2,4])
 
+            image = np.rot90(image, k=num_rots, axes=(1, 0))
             is_grey = np.issubdtype(image.dtype, np.int8) or np.issubdtype(
                 image.dtype, np.uint8
             )
+
             aug = Augmentation(is_grey)
             image = aug.image_augmentation(image)
-            # resize the image to standard size
+
+        # resize the image to standard size
         if not jitter:
-            image = np.array(
-                Image.fromarray(image).resize(
-                    (self.config["IMAGE_H"], self.config["IMAGE_W"]),
-                    resample=Image.BILINEAR,
-                )
-            )
+            image = resize(image, (self.config["IMAGE_W"], self.config["IMAGE_H"]), copy=True)
 
         if self.imgdepth == 3:
             image = image[:, :, ::-1]
 
         # fix objects's position and size and check region
         obj_is_region = []
-        # h = 4096.0
-        # w = 4096.0
-
         original_width, original_height = train_instance["img_size"]
         if (
             int(original_width - w) != 0 or int(original_height - h) != 0
-        ) and region is None:
-            """
-                        pass
-            if region is None:
-            """
+        ) and patch_mode is False:
             # In case of "fast filtering", we need the original image size here
-            w, h = train_instance["img_size"]
+            w = original_width
+            h = original_height
+            if region is not None:
+                # In case of mixed-sizes  we have to set it to the size of the original cut-off region...
+                w = region[1].stop - region[1].start
+                h = region[0].stop - region[0].start
             if jitter:
                 offx = offx * w / float(self.config["IMAGE_W"])
                 offy = offy * h / float(self.config["IMAGE_H"])
 
+
         for obj in all_objs:
             if region is None:
                 is_in_region = True
@@ -352,8 +389,8 @@ class PatchwiseBatchGenerator(Sequence):
                 region_off_y = 0
 
                 if region is not None:
-                    region_off_x = region[0].start  # * w/float(self.config["IMAGE_W"])
-                    region_off_y = region[1].start  # * h/float(self.config["IMAGE_H"])
+                    region_off_x = region[1].start  # * w/float(self.config["IMAGE_W"])
+                    region_off_y = region[0].start  # * h/float(self.config["IMAGE_H"])
 
                 for attr in ["xmin", "xmax"]:
                     obj[attr] = obj[attr] - region_off_x
