@@ -182,13 +182,6 @@ def create_parser(parser):
         help="Use it if you want to deactivate data augmentation during training.",
     )
 
-    deexsp_group.add_argument(
-        "--use_multithreading",
-        action="store_true",
-        default=distutils.strtobool(os.environ["CRYOLO_USE_MULTITHREADING"]),
-        help="If python multiprocessing leads to problems during training (e.g. freezing, dying workers) use multithreading instead of multiprocessing.",
-    )
-
 def get_parser():
     """
     Creates parser
@@ -328,8 +321,6 @@ def main(args=None):
 
     util.check_for_updates()
 
-    print("START METHOD:", multiprocessing.get_start_method())
-
     if args is None:
         parser = get_parser()
         args = parser.parse_args()
@@ -373,7 +364,7 @@ def main(args=None):
         set_session(tf.Session(config=config))
 
     early_stop = int(args.early)
-    do_multiprocessing = args.use_multithreading == False
+    do_multiprocessing = False
     warm_restarts = args.warm_restarts
 
     do_augmentation = not args.skip_augmentation
@@ -757,7 +748,7 @@ def main(args=None):
 
 
 if __name__ == "__main__":
-    start_method = "spawn"
+    start_method = "fork"
     try:
         os_start_method = os.environ["CRYOLO_MP_START"]
 
