@@ -23,7 +23,7 @@ import sys
 
 from . import imagereader
 import urllib.request, json
-
+from sklearn import mixture
 
 class Filament:
     """
@@ -239,9 +239,36 @@ def resample_filament(filament, distance):
 
         return new_filament
     return None
+def get_cryolo_mode(size_distr, config_input_size):
+    square_data_available = any([size[0] == size[1] for size in size_distr])
+    non_square_data_available = any([size[0] != size[1] for size in size_distr])
 
+    if type(config_input_size) == list:
+        return "FIXED-SIZE"
 
-def get_anchors(config, path_weights=None, image_size=None):
+    if (square_data_available and non_square_data_available) or \
+            (non_square_data_available and len(size_distr)>1):
+        return "MIXED-ASPECT-RATIO"
+
+    if square_data_available and not non_square_data_available:
+        return "SQUARE"
+
+    if not square_data_available and non_square_data_available and len(size_distr)==1:
+        return "NON-SQUARE"
+
+def get_anchors(config, path_weights=None, image_sizes=None):
+    '''
+    Calculates the anchors.
+    When fixed size mode ...
+    When non-square mode...
+    When square-mode ...
+    When mixed-size mode
+
+    :param config:
+    :param path_weights:
+    :param image_size:
+    :return:
+    '''
     import cryolo.config_tools as config_tools
 
     is_single_anchor = len(config["model"]["anchors"]) == 2
@@ -263,15 +290,34 @@ def get_anchors(config, path_weights=None, image_size=None):
 
     if is_single_anchor and anchors is None:
 
-        if image_size:
-            img_width = float(image_size[0]) / num_patches
-            img_height = float(image_size[1]) / num_patches
+        # if single size, use the it as given
+        # if multi-size, use the average length of the short side
+        #
+        if len(image_sizes) == 1:
+            img_width = float(image_sizes[0][1]) / num_patches
+            img_height = float(image_sizes[0][0]) / num_patches
         else:
-            img_width = 4096.0 / num_patches
-            img_height = 4096.0 / num_patches
+            sum_size = 0
+            sum_num_images = 0
+            for img_info in image_sizes:
+                if img_info[0]>img_info[1]:
+                    sum_size += img_info[1] * img_info[2]
+                else:
+                    sum_size += img_info[0] * img_info[2]
+                sum_num_images += img_info[2]
+            mean_size = int(sum_size/sum_num_images)
+            img_width = mean_size
+            img_height = mean_size
+
+        #if image_sizes:
+
+        #    img_width = float(image_size[0]) / num_patches
+        #    img_height = float(image_size[1]) / num_patches
+        #else:
+        #    img_width = 4096.0 / num_patches
+        #    img_height = 4096.0 / num_patches
         cell_w = img_width / grid_w
         cell_h = img_height / grid_h
-
         box_width, box_height = config_tools.get_box_size(config)
 
         anchor_width = 1.0 * box_width / cell_w
@@ -298,6 +344,49 @@ def box_squared_distance(boxa, boxb):
     """
     return np.square(boxa.x - boxb.x) + np.square(boxa.y - boxb.y)
 
+def get_normalization_func(normalization):
+    if normalization == "STANDARD":
+        norm_func = normalize
+    elif normalization == "GMM":
+        norm_func = normalize_gmm
+    else:
+        print("Normalization option ", normalization, "unknown. Check your config file.")
+        import sys
+        sys.exit(1)
+    return norm_func
+
+def normalize_gmm(image, margin_size=0):
+
+
+    if margin_size < 0 or margin_size > 1:
+        print("Normalization has to be between 0 and 1.")
+        if margin_size < 0:
+            margin_size = 0
+        else:
+            margin_size = 1
+        print("Set it to", margin_size)
+
+    if not np.issubdtype(image.dtype, np.float32):
+        image = image.astype(np.float32)
+    mask = np.s_[
+        int(image.shape[0] * (margin_size)) : int(image.shape[0] * (1 - margin_size)),
+        int(image.shape[1] * margin_size) : int(image.shape[1] * (1 - margin_size)),
+    ]
+
+    clf = mixture.GaussianMixture(n_components=2, covariance_type='diag')
+    clf.fit(np.expand_dims(image[mask].ravel()[::15], axis=1))
+    mean = None
+    var = None
+    if (clf.means_[1, 0]>clf.means_[0, 0]):
+        mean = clf.means_[1, 0]
+        var = clf.covariances_[1, 0]
+    else:
+        mean = clf.means_[0, 0]
+        var = clf.covariances_[0, 0]
+    #print(mean,var,np.sqrt(var))
+    image = (image - mean) / (2*3*np.sqrt(var))
+
+    return image
 
 def normalize(image, margin_size=0):
     """
@@ -527,7 +616,7 @@ def check_for_updates():
                 print("Latest version:\t\t", latest_version)
                 print(
                     "More information here:\n",
-                    "http://sphire.mpg.de/wiki/doku.php?id=pipeline:window:cryolo",
+                    "https://cryolo.readthedocs.io/en/latest/changes.html",
                 )
                 print("###############################################")
     except Exception as e:
